/* MicroC/OS-II definitions */
#include "includes.h"

#include <math.h>
#include "altera_avalon_pio_regs.h"
#include "system.h"
#include "ipport.h"
#include "menu.h"
#include "demo_control.h"
#include "triple_speed_ethernet_regs.h"

#include <sys/alt_flash.h>

#include "priv/alt_file.h"
#include "altera_ro_zipfs.h"

#define NO_STDFILE_CALLS 1
#include "vfsfiles.h"

int display_build(void * pio);
int dump_mac_info(void * pio);
int ls_rozipfs(void * pio);
int ro_cat(void * pio);
int cp_ro2vfs(void * pio);
int vfs_cat(void * pio);
int flash_rozipfs(void * pio);

//
//	Menu definition for InterNiche menu command environment
//
struct menu_op demo_menu[] = {
	{ "demo          ",	stooges,		"Demo menu" } ,
	{ "build         ",	display_build,	"Built on " __DATE__ " at " __TIME__} ,
	{ "dump_mac_info ",	dump_mac_info,	"Dump the MAC registers." } ,
	{ "ls_rozipfs    ",	ls_rozipfs,		"list the rozipfs filesystem" } ,
	{ "ro_cat        ",	ro_cat,			"catenate a rozipfs file to the output stream" } ,
	{ "vfs_cat       ",	vfs_cat,		"catenate a vfs file to the output stream" } ,
	{ "cp_ro2vfs     ",	cp_ro2vfs,		"copy a file from rozipfs to vfs" } ,
	{ "flash_rozipfs ",	flash_rozipfs,	"program a file from VFS into flash at rozipfs location" } ,
	{ NULL },
};

//
//	Initialize our menu and add it to the menu system so we can invoke our menu
//	commands from a console or telnet session.
//
void demo_init(void) {

	alt_dev* rozipfs_dev;

	// turn off user leds
	IOWR_ALTERA_AVALON_PIO_DATA(USER_LED_PIO_8OUT_BASE, 0xFF);
	
	// install menu applications
	if (install_menu(demo_menu)) {
		ns_printf(0, "\nDemo Menu failed to install...\n");
	} else {
		display_build(NULL);
		ns_printf(0, "Demo Menu installed and ready to use...\n");
		ns_printf(0, "Type \"help\" and \"help demo\" for more information.\n\n");
		
		ns_printf(0, "VFS installed at \"%s\"\n\n", vfs_root_path);
		
		rozipfs_dev = alt_find_file (ALTERA_RO_ZIPFS_NAME);
		
		if(NULL == rozipfs_dev) {
			ns_printf(0, "ROZIPFS was not discovered...\n");
		} else {
			ns_printf(0, "ROZIPFS installed at \"%s\"\n\n", ALTERA_RO_ZIPFS_NAME);
		}
	}
}

//
//	This menu command displays the build date and time of this application
//
int display_build(void * pio) {

    ns_printf(pio,"\nBuilt on %s at %s\n\n", __DATE__, __TIME__);

    return 0;
}

//
//	This menu command displays the control and status registers for the TSE MAC.
//	
int dump_mac_info(void * pio) {
	
	// allocate a structure to hold the current register values
	np_tse_mac MY_MAC;
	
	// read all the registers
	MY_MAC.REV								= IORD_ALTERA_TSEMAC_REV(TSE_MAC_BASE);
	MY_MAC.SCRATCH							= IORD_ALTERA_TSEMAC_SCRATCH(TSE_MAC_BASE);
	MY_MAC.COMMAND_CONFIG					= IORD_ALTERA_TSEMAC_CMD_CONFIG(TSE_MAC_BASE);
	MY_MAC.MAC_0							= IORD_ALTERA_TSEMAC_MAC_0(TSE_MAC_BASE);
	MY_MAC.MAC_1							= IORD_ALTERA_TSEMAC_MAC_1(TSE_MAC_BASE);
	MY_MAC.FRM_LENGTH						= IORD_ALTERA_TSEMAC_FRM_LENGTH(TSE_MAC_BASE);
	MY_MAC.PAUSE_QUANT						= IORD_ALTERA_TSEMAC_PAUSE_QUANT(TSE_MAC_BASE);
	MY_MAC.RX_SECTION_EMPTY					= IORD_ALTERA_TSEMAC_RX_SECTION_EMPTY(TSE_MAC_BASE);
	MY_MAC.RX_SECTION_FULL					= IORD_ALTERA_TSEMAC_RX_SECTION_FULL(TSE_MAC_BASE);
	MY_MAC.TX_SECTION_EMPTY					= IORD_ALTERA_TSEMAC_TX_SECTION_EMPTY(TSE_MAC_BASE);
	MY_MAC.TX_SECTION_FULL					= IORD_ALTERA_TSEMAC_TX_SECTION_FULL(TSE_MAC_BASE);
	MY_MAC.RX_ALMOST_EMPTY					= IORD_ALTERA_TSEMAC_RX_ALMOST_EMPTY(TSE_MAC_BASE);
	MY_MAC.RX_ALMOST_FULL					= IORD_ALTERA_TSEMAC_RX_ALMOST_FULL(TSE_MAC_BASE);
	MY_MAC.TX_ALMOST_EMPTY					= IORD_ALTERA_TSEMAC_TX_ALMOST_EMPTY(TSE_MAC_BASE);
	MY_MAC.TX_ALMOST_FULL					= IORD_ALTERA_TSEMAC_TX_ALMOST_FULL(TSE_MAC_BASE);
	MY_MAC.MDIO_ADDR0						= IORD_ALTERA_TSEMAC_MDIO_ADDR0(TSE_MAC_BASE);
	MY_MAC.MDIO_ADDR1						= IORD_ALTERA_TSEMAC_MDIO_ADDR1(TSE_MAC_BASE);
	MY_MAC.REG_STAT							= IORD_ALTERA_TSEMAC_REG_STAT(TSE_MAC_BASE);
	MY_MAC.TX_IPG_LENGTH					= IORD_ALTERA_TSEMAC_TX_IPG_LENGTH(TSE_MAC_BASE);
	MY_MAC.aMACID_1							= IORD_ALTERA_TSEMAC_A_MACID_1(TSE_MAC_BASE);
	MY_MAC.aMACID_2							= IORD_ALTERA_TSEMAC_A_MACID_2(TSE_MAC_BASE);
	MY_MAC.aFramesTransmittedOK				= IORD_ALTERA_TSEMAC_A_FRAMES_TX_OK(TSE_MAC_BASE);
	MY_MAC.aFramesReceivedOK				= IORD_ALTERA_TSEMAC_A_FRAMES_RX_OK(TSE_MAC_BASE);
	MY_MAC.aFramesCheckSequenceErrors		= IORD_ALTERA_TSEMAC_A_FRAME_CHECK_SEQ_ERRS(TSE_MAC_BASE);
	MY_MAC.aAlignmentErrors					= IORD_ALTERA_TSEMAC_A_ALIGNMENT_ERRS(TSE_MAC_BASE);
	MY_MAC.aOctetsTransmittedOK				= IORD_ALTERA_TSEMAC_A_OCTETS_TX_OK(TSE_MAC_BASE);
	MY_MAC.aOctetsReceivedOK				= IORD_ALTERA_TSEMAC_A_OCTETS_RX_OK(TSE_MAC_BASE);
	MY_MAC.aTxPAUSEMACCtrlFrames			= IORD_ALTERA_TSEMAC_A_TX_PAUSE_MAC_CTRL_FRAMES(TSE_MAC_BASE);
	MY_MAC.aRxPAUSEMACCtrlFrames			= IORD_ALTERA_TSEMAC_A_RX_PAUSE_MAC_CTRL_FRAMES(TSE_MAC_BASE);
	MY_MAC.ifInErrors						= IORD_ALTERA_TSEMAC_IF_IN_ERRORS(TSE_MAC_BASE);
	MY_MAC.ifOutErrors						= IORD_ALTERA_TSEMAC_IF_OUT_ERRORS(TSE_MAC_BASE);
	MY_MAC.ifInUcastPkts					= IORD_ALTERA_TSEMAC_IF_IN_UCAST_PKTS(TSE_MAC_BASE);
	MY_MAC.ifInMulticastPkts				= IORD_ALTERA_TSEMAC_IF_IN_MULTICAST_PKTS(TSE_MAC_BASE);
	MY_MAC.ifInBroadcastPkts				= IORD_ALTERA_TSEMAC_IF_IN_BROADCAST_PKTS(TSE_MAC_BASE);
	MY_MAC.ifOutDiscards					= IORD_ALTERA_TSEMAC_IF_OUT_DISCARDS(TSE_MAC_BASE);
	MY_MAC.ifOutUcastPkts					= IORD_ALTERA_TSEMAC_IF_OUT_UCAST_PKTS(TSE_MAC_BASE);
	MY_MAC.ifOutMulticastPkts				= IORD_ALTERA_TSEMAC_IF_OUT_MULTICAST_PKTS(TSE_MAC_BASE);
	MY_MAC.ifOutBroadcastPkts				= IORD_ALTERA_TSEMAC_IF_OUT_BROADCAST_PKTS(TSE_MAC_BASE);
	MY_MAC.etherStatsDropEvent				= IORD_ALTERA_TSEMAC_ETHER_STATS_DROP_EVENTS(TSE_MAC_BASE);
	MY_MAC.etherStatsOctets					= IORD_ALTERA_TSEMAC_ETHER_STATS_OCTETS(TSE_MAC_BASE);
	MY_MAC.etherStatsPkts					= IORD_ALTERA_TSEMAC_ETHER_STATS_PKTS(TSE_MAC_BASE);
	MY_MAC.etherStatsUndersizePkts			= IORD_ALTERA_TSEMAC_ETHER_STATS_UNDERSIZE_PKTS(TSE_MAC_BASE);
	MY_MAC.etherStatsOversizePkts			= IORD_ALTERA_TSEMAC_ETHER_STATS_OVERSIZE_PKTS(TSE_MAC_BASE);
	MY_MAC.etherStatsPkts64Octets			= IORD_ALTERA_TSEMAC_ETHER_STATS_PKTS_64_OCTETS(TSE_MAC_BASE);
	MY_MAC.etherStatsPkts65to127Octets		= IORD_ALTERA_TSEMAC_ETHER_STATS_PKTS_65_TO_127_OCTETS(TSE_MAC_BASE);
	MY_MAC.etherStatsPkts128to255Octets		= IORD_ALTERA_TSEMAC_ETHER_STATS_PKTS_128_TO_255_OCTETS(TSE_MAC_BASE);
	MY_MAC.etherStatsPkts256to511Octets		= IORD_ALTERA_TSEMAC_ETHER_STATS_PKTS_256_TO_511_OCTETS(TSE_MAC_BASE);
	MY_MAC.etherStatsPkts512to1023Octets	= IORD_ALTERA_TSEMAC_ETHER_STATS_PKTS_512_TO_1023_OCTETS(TSE_MAC_BASE);
	MY_MAC.etherStatsPkts1024to1518Octets	= IORD_ALTERA_TSEMAC_ETHER_STATS_PKTS_1024_TO_1518_OCTETS(TSE_MAC_BASE);
	MY_MAC.etherStatsPkts1519toXOctets						= IORD_ALTERA_TSEMAC_ETHER_STATS_PKTS_1519_TO_X_OCTETS(TSE_MAC_BASE);
	MY_MAC.etherStatsJabbers						= IORD_ALTERA_TSEMAC_ETHER_STATS_JABBERS(TSE_MAC_BASE);
	MY_MAC.etherStatsFragments						= IORD_ALTERA_TSEMAC_ETHER_STATS_FRAGMENTS(TSE_MAC_BASE);
	MY_MAC.TX_CMD_STAT						= IORD_ALTERA_TSEMAC_TX_CMD_STAT(TSE_MAC_BASE);
	MY_MAC.RX_CMD_STAT						= IORD_ALTERA_TSEMAC_RX_CMD_STAT(TSE_MAC_BASE);
	
	ns_printf(pio,"********************* CURRENT MAC STATS *********************\n\n", MY_MAC.REV);
	
	ns_printf(pio,"                           REV = 0x%08X\n", MY_MAC.REV);
	ns_printf(pio,"                       SCRATCH = 0x%08X\n", MY_MAC.SCRATCH);
	ns_printf(pio,"                COMMAND_CONFIG = 0x%08X\n", MY_MAC.COMMAND_CONFIG);
	ns_printf(pio,"                         MAC_0 = 0x%08X\n", MY_MAC.MAC_0);
	ns_printf(pio,"                         MAC_1 = 0x%08X\n", MY_MAC.MAC_1);
	ns_printf(pio,"                    FRM_LENGTH = 0x%08X  %uu\n", MY_MAC.FRM_LENGTH, MY_MAC.FRM_LENGTH);
	ns_printf(pio,"                   PAUSE_QUANT = 0x%08X\n", MY_MAC.PAUSE_QUANT);
	ns_printf(pio,"              RX_SECTION_EMPTY = 0x%08X\n", MY_MAC.RX_SECTION_EMPTY);
	ns_printf(pio,"               RX_SECTION_FULL = 0x%08X\n", MY_MAC.RX_SECTION_FULL);
	ns_printf(pio,"              TX_SECTION_EMPTY = 0x%08X\n", MY_MAC.TX_SECTION_EMPTY);
	ns_printf(pio,"               TX_SECTION_FULL = 0x%08X\n", MY_MAC.TX_SECTION_FULL);
	ns_printf(pio,"               RX_ALMOST_EMPTY = 0x%08X\n", MY_MAC.RX_ALMOST_EMPTY);
	ns_printf(pio,"                RX_ALMOST_FULL = 0x%08X\n", MY_MAC.RX_ALMOST_FULL);
	ns_printf(pio,"               TX_ALMOST_EMPTY = 0x%08X\n", MY_MAC.TX_ALMOST_EMPTY);
	ns_printf(pio,"                TX_ALMOST_FULL = 0x%08X\n", MY_MAC.TX_ALMOST_FULL);
	ns_printf(pio,"                    MDIO_ADDR0 = 0x%08X\n", MY_MAC.MDIO_ADDR0);
	ns_printf(pio,"                    MDIO_ADDR1 = 0x%08X\n", MY_MAC.MDIO_ADDR1);
	ns_printf(pio,"                      REG_STAT = 0x%08X\n", MY_MAC.REG_STAT);
	ns_printf(pio,"                 TX_IPG_LENGTH = 0x%08X  %uu\n", MY_MAC.TX_IPG_LENGTH, MY_MAC.TX_IPG_LENGTH);
	ns_printf(pio,"                      aMACID_1 = 0x%08X\n", MY_MAC.aMACID_1);
	ns_printf(pio,"                      aMACID_2 = 0x%08X\n", MY_MAC.aMACID_2);
	ns_printf(pio,"          aFramesTransmittedOK = 0x%08X  %uu\n", MY_MAC.aFramesTransmittedOK, MY_MAC.aFramesTransmittedOK);
	ns_printf(pio,"             aFramesReceivedOK = 0x%08X  %uu\n", MY_MAC.aFramesReceivedOK, MY_MAC.aFramesReceivedOK);
	ns_printf(pio,"    aFramesCheckSequenceErrors = 0x%08X  %uu\n", MY_MAC.aFramesCheckSequenceErrors, MY_MAC.aFramesCheckSequenceErrors);
	ns_printf(pio,"              aAlignmentErrors = 0x%08X  %uu\n", MY_MAC.aAlignmentErrors, MY_MAC.aAlignmentErrors);
	ns_printf(pio,"          aOctetsTransmittedOK = 0x%08X  %uu\n", MY_MAC.aOctetsTransmittedOK, MY_MAC.aOctetsTransmittedOK);
	ns_printf(pio,"             aOctetsReceivedOK = 0x%08X  %uu\n", MY_MAC.aOctetsReceivedOK, MY_MAC.aOctetsReceivedOK);
	ns_printf(pio,"         aTxPAUSEMACCtrlFrames = 0x%08X  %uu\n", MY_MAC.aTxPAUSEMACCtrlFrames, MY_MAC.aTxPAUSEMACCtrlFrames);
	ns_printf(pio,"         aRxPAUSEMACCtrlFrames = 0x%08X  %uu\n", MY_MAC.aRxPAUSEMACCtrlFrames, MY_MAC.aRxPAUSEMACCtrlFrames);
	ns_printf(pio,"                    ifInErrors = 0x%08X  %uu\n", MY_MAC.ifInErrors, MY_MAC.ifInErrors);
	ns_printf(pio,"                   ifOutErrors = 0x%08X  %uu\n", MY_MAC.ifOutErrors, MY_MAC.ifOutErrors);
	ns_printf(pio,"                 ifInUcastPkts = 0x%08X  %uu\n", MY_MAC.ifInUcastPkts, MY_MAC.ifInUcastPkts);
	ns_printf(pio,"             ifInMulticastPkts = 0x%08X  %uu\n", MY_MAC.ifInMulticastPkts, MY_MAC.ifInMulticastPkts);
	ns_printf(pio,"             ifInBroadcastPkts = 0x%08X  %uu\n", MY_MAC.ifInBroadcastPkts, MY_MAC.ifInBroadcastPkts);
	ns_printf(pio,"                 ifOutDiscards = 0x%08X  %uu\n", MY_MAC.ifOutDiscards, MY_MAC.ifOutDiscards);
	ns_printf(pio,"                ifOutUcastPkts = 0x%08X  %uu\n", MY_MAC.ifOutUcastPkts, MY_MAC.ifOutUcastPkts);
	ns_printf(pio,"            ifOutMulticastPkts = 0x%08X  %uu\n", MY_MAC.ifOutMulticastPkts, MY_MAC.ifOutMulticastPkts);
	ns_printf(pio,"            ifOutBroadcastPkts = 0x%08X  %uu\n", MY_MAC.ifOutBroadcastPkts, MY_MAC.ifOutBroadcastPkts);
	ns_printf(pio,"           etherStatsDropEvent = 0x%08X  %uu\n", MY_MAC.etherStatsDropEvent, MY_MAC.etherStatsDropEvent);
	ns_printf(pio,"              etherStatsOctets = 0x%08X  %uu\n", MY_MAC.etherStatsOctets, MY_MAC.etherStatsOctets);
	ns_printf(pio,"                etherStatsPkts = 0x%08X  %uu\n", MY_MAC.etherStatsPkts, MY_MAC.etherStatsPkts);
	ns_printf(pio,"       etherStatsUndersizePkts = 0x%08X  %uu\n", MY_MAC.etherStatsUndersizePkts, MY_MAC.etherStatsUndersizePkts);
	ns_printf(pio,"        etherStatsOversizePkts = 0x%08X  %uu\n", MY_MAC.etherStatsOversizePkts, MY_MAC.etherStatsOversizePkts);
	ns_printf(pio,"        etherStatsPkts64Octets = 0x%08X  %uu\n", MY_MAC.etherStatsPkts64Octets, MY_MAC.etherStatsPkts64Octets);
	ns_printf(pio,"   etherStatsPkts65to127Octets = 0x%08X  %uu\n", MY_MAC.etherStatsPkts65to127Octets, MY_MAC.etherStatsPkts65to127Octets);
	ns_printf(pio,"  etherStatsPkts128to255Octets = 0x%08X  %uu\n", MY_MAC.etherStatsPkts128to255Octets, MY_MAC.etherStatsPkts128to255Octets);
	ns_printf(pio,"  etherStatsPkts256to511Octets = 0x%08X  %uu\n", MY_MAC.etherStatsPkts256to511Octets, MY_MAC.etherStatsPkts256to511Octets);
	ns_printf(pio," etherStatsPkts512to1023Octets = 0x%08X  %uu\n", MY_MAC.etherStatsPkts512to1023Octets, MY_MAC.etherStatsPkts512to1023Octets);
	ns_printf(pio,"etherStatsPkts1024to1518Octets = 0x%08X  %uu\n", MY_MAC.etherStatsPkts1024to1518Octets, MY_MAC.etherStatsPkts1024to1518Octets);
	ns_printf(pio,"   etherStatsPkts1519toXOctets = 0x%08X  %uu\n", MY_MAC.etherStatsPkts1519toXOctets, MY_MAC.etherStatsPkts1519toXOctets);
	ns_printf(pio,"             etherStatsJabbers = 0x%08X  %uu\n", MY_MAC.etherStatsJabbers, MY_MAC.etherStatsJabbers);
	ns_printf(pio,"           etherStatsFragments = 0x%08X  %uu\n", MY_MAC.etherStatsFragments, MY_MAC.etherStatsFragments);
	ns_printf(pio,"                   TX_CMD_STAT = 0x%08X\n", MY_MAC.TX_CMD_STAT);
	ns_printf(pio,"                   RX_CMD_STAT = 0x%08X\n", MY_MAC.RX_CMD_STAT);
	
	ns_printf(pio,"\n");
	
	return 0;
}

//
//	This menu command lists the rozip file system
//
//	This functionality was hacked out of the altera_ro_zipfs.c file since there
//	is not a formal API provided by ROZIPFS to list the file system.
//

#define ZIP_FILE_RECORD             0x04034b50
#define ZIP_FILE_HEADER_SIZE        30
#define ZIP_FILE_EXTRA_LEN          28

#define ZIP_DIRECTORY_RECORD        0x02014b50
#define ZIP_DIR_UNCOMPRESSED_LEN    24
#define ZIP_DIR_NAME_LEN            28
#define ZIP_DIR_EXTRA_LEN           30
#define ZIP_DIR_COMMENT_LEN         32
#define ZIP_DIR_OFFSET              42
#define ZIP_DIR_HEADER_SIZE         46

#define ZIP_END_OF_DIRECTORY_RECORD 0x06054b50

/*
* read_word
*
* reads a word from an address in flash
* this is necessary as we can't do 
* unaligned from an unaligned boundary in Nios II
*/
static alt_u32 read_word(alt_u8* current_ptr)
{
  return (alt_u32)(*current_ptr | 
         (*(current_ptr+1) << 8) | 
         (*(current_ptr+2) << 16) |
         (*(current_ptr+3) << 24 ));
}

/*
* read_half_word
*
* reads a half word from an address in flash
* this is necessary as we can't do 
* unaligned from an unaligned boundary in Nios II
*/
static alt_u16 read_half_word(alt_u8* current_ptr)
{
  return (alt_u16)(*current_ptr | 
         (*(current_ptr+1) << 8));
}

int ls_rozipfs(void * pio) {

	alt_dev* rozipfs_dev;
	
	rozipfs_dev = alt_find_file (ALTERA_RO_ZIPFS_NAME);
	
	if(NULL == rozipfs_dev) {
		ns_printf(pio,"\nCannot locate filesystem \"%s\"\n\n", ALTERA_RO_ZIPFS_NAME);
	} else {
		alt_ro_zipfs_dev* dev = (alt_ro_zipfs_dev*)(rozipfs_dev);
		alt_u32 token;
		alt_u8* current_ptr = dev->directory;
		alt_u16 name_len, extra_len, comment_len;
		alt_u32 offset;
		char filename[256];
		
		current_ptr = dev->directory;
		
		while(1)
		{
		  token = read_word(current_ptr);
		
		  if(token != ZIP_DIRECTORY_RECORD)
		  {
		    break;
		  }
		
		  offset = read_word(current_ptr + ZIP_DIR_OFFSET);
		  name_len = read_half_word(current_ptr + ZIP_DIR_NAME_LEN);
		  extra_len = read_half_word(current_ptr + ZIP_DIR_EXTRA_LEN);
		  comment_len = read_half_word(current_ptr + ZIP_DIR_COMMENT_LEN);
		
		  strncpy(filename, (const char *)(current_ptr+ZIP_DIR_HEADER_SIZE), name_len);
		  filename[name_len] = '\0';
		  ns_printf(pio,"%s/%s\n", ALTERA_RO_ZIPFS_NAME, filename);

		  current_ptr += ZIP_DIR_HEADER_SIZE + name_len + extra_len + comment_len;
		}
	}

    return 0;
}

//
//	This menu command will catentate a file to stdout
//
int ro_cat(void * pio) {
	
	char *arg1 = (char *)NULL;
	GEN_IO io = (GEN_IO)(pio);
	FILE* fd;
	
	/* extract args from command buffer */
	if (io != NULL)
		arg1 = nextarg((io->inbuf));
	if((arg1 == NULL) || (*arg1 == '\0')) {
		ns_printf(pio, "invalid arguments to command...\ncat <filename>\n");
		return -1;
	}
	
	fd = fopen(arg1, "r");
	if(NULL == fd) {
		ns_printf(pio, "\nUnable to open file \"%s\"\n", arg1);
	} else {
		char buffer[256];
		char* buf_ptr;
		do {
			buf_ptr = fgets(buffer, 256, fd);
			if(NULL != buf_ptr) {
				ns_printf(pio, "%s", buf_ptr);
			}
		}while(NULL != buf_ptr);
		fclose(fd);
	}
	
	return 0;
}

//
//	This menu command will catentate a file to stdout
//
int vfs_cat(void * pio) {
	
	char *arg1 = (char *)NULL;
	GEN_IO io = (GEN_IO)(pio);
	VFILE* vfd;
	
	/* extract args from command buffer */
	if (io != NULL)
		arg1 = nextarg((io->inbuf));
	if((arg1 == NULL) || (*arg1 == '\0')) {
		ns_printf(pio, "invalid arguments to command...\ncat <filename>\n");
		return -1;
	}
	
	vfd = vfopen(arg1, "r");
	if(NULL == vfd) {
		ns_printf(pio, "\nUnable to open file \"%s\"\n", arg1);
	} else {
		char buffer[256];
		char* buf_ptr;
		do {
			buf_ptr = vfgets(buffer, 256, vfd);
			if(NULL != buf_ptr) {
				ns_printf(pio, "%s", buf_ptr);
			}
		}while(NULL != buf_ptr);
		vfclose(vfd);
	}
	
	return 0;
}

//
//	This menu command will copy a file from ROZIPFS to VFS
//
int cp_ro2vfs(void * pio) {
	
	char *arg1 = (char *)NULL;
	GEN_IO io = (GEN_IO)(pio);
	FILE* fd;
	VFILE* vfd;
	
	/* extract args from command buffer */
	if (io != NULL)
		arg1 = nextarg((io->inbuf));
	if((arg1 == NULL) || (*arg1 == '\0')) {
		ns_printf(pio, "invalid arguments to command...\ncp_ro2vfs <filename>\n");
		return -1;
	}
	
	fd = fopen(arg1, "r");
	if(NULL == fd) {
		ns_printf(pio, "\nUnable to open ROZIPFS file \"%s\"\n", arg1);
	} else {
		arg1 = strrchr(arg1, '/');
		arg1++;
		vfd = vfopen(arg1, "w");
		if(NULL == vfd) {
			ns_printf(pio, "\nUnable to open VFS file \"%s\"\n", arg1);
			fclose(fd);
		} else {
			char buffer[256];
			size_t count;
			do {
				count = fread(buffer, sizeof(char), 256, fd);
				if(0x00 != count) {
					vfwrite(buffer, sizeof(char), count, vfd);
				}
			}while(0x00 != count);
			fclose(fd);
			vfclose(vfd);
		}
	}
	
	return 0;
}

//
//	This menu command will catentate a file to stdout
//
int flash_rozipfs(void * pio) {
	
	char *arg1 = (char *)NULL;
	GEN_IO io = (GEN_IO)(pio);
	VFILE* vfd;
	
	/* extract args from command buffer */
	if (io != NULL)
		arg1 = nextarg((io->inbuf));
	if((arg1 == NULL) || (*arg1 == '\0')) {
		ns_printf(pio, "invalid arguments to command...\nflash_rozipfs <filename>\n");
		return -1;
	}
	
	vfd = vfopen(arg1, "rb");
	if(NULL == vfd) {
		ns_printf(pio, "\nUnable to open file \"%s\"\n", arg1);
	} else {
		char buffer[4096];
		size_t count;
		alt_flash_fd* flash_fd;
		int flash_offset = 0;
		
		
		flash_fd = alt_flash_open_dev(CFI_FLASH_64M_NAME);
		
		if(NULL == flash_fd) {
			ns_printf(pio, "\nUnable to open flash device \"%s\"\n", CFI_FLASH_64M_NAME);
		} else {
		
			do {
				count = vfread(buffer, sizeof(char), 4096, vfd);
				if(0x00 != count) {
					if (
						alt_write_flash(
							flash_fd,		// alt_flash_fd* fd,
							flash_offset,	// int offset,
							buffer,			// const void* src_addr,
							count			// int length
							)
						) {
						ns_printf(pio, "\nError writing to flash offset 0x%08X\n", flash_offset);
					}
					flash_offset += count;
				}
			}while(0x00 != count);
			
			alt_flash_close_dev(flash_fd);
			vfclose(vfd);
		}
	}
	
	return 0;
}
