#!/bin/bash

# create a file name for an archive to store patch source files in after we create their patch file
SOURCE_ARCHIVE="patch_sources.tar"

# locate patch source files in this directory
FILES_TO_PATCH=$(find . -name '*.h' -o -name '*.c' -o -name '*.S')

if [ -z "${FILES_TO_PATCH}" ] ; then

	echo ""
	echo "There are no patch files to create..."
	echo ""
	exit 1

else
	
	# obtain the basename of this directory so we can exclude it in future searches
	CUR_DIR_BASE=$(basename $(pwd))

	for NEW_FILE in ${FILES_TO_PATCH} ; do

		# obtain the basename of the file so we can use it in the next search
		FILE_NAME=$(basename ${NEW_FILE})
		
		# locate the file that this will patch
		OLD_FILE=$(find .. -wholename *${CUR_DIR_BASE}* -prune -o -name ${FILE_NAME} -print)
		
		if [ -z "${OLD_FILE}" ] ; then
		
			echo ""
			echo "ERROR - could not find old file to patch for file \"${FILE_NAME}\""
			echo ""
			exit 2
			
		else
		
			# make sure we only found one file to patch in the source tree
			for CHECK_OLD_FILE in ${OLD_FILE} ; do
			
				if [ "${CHECK_OLD_FILE}" != "${OLD_FILE}" ] ; then
				
					echo ""
					echo "ERROR - found multiple old files to patch for file \"${FILE_NAME}\""
					echo "These files were found:"
					echo "${OLD_FILE}"
					echo ""
					exit 3
					
				fi
				
			done
			
			echo "Creating patch for file \"${OLD_FILE}\""
			
			# make sure both files are unix format before diff'ing them
			dos2unix ${NEW_FILE} 2> /dev/null
			dos2unix ${OLD_FILE} 2> /dev/null
			
			# create the diff patch file
			diff -urN ${OLD_FILE} ${NEW_FILE} > ${FILE_NAME}.patch
			
			# archive the patch source file
			tar --remove-files -rf ${SOURCE_ARCHIVE} ${NEW_FILE}
			
		fi

	done

fi

exit 0
